package de.unibi.citec.fit.javasamples.strategies;

import de.unibi.citec.fit.javasamples.SampleSolution;
import java.util.ArrayList;

/**
 * own method, string-based, split-first approach with comments.
 *
 * @author Benjamin Paassen - bpaassen(at)techfak.uni-bielefeld.de
 */
public class ImplAAA1 implements SampleSolution {

	/**
	 * Checks if all words of the given sentence are palindromes.
	 * 
	 * @param sentence a sentence with words separated by a single whitespace.
	 * @return true if all words are palindromes and false otherwise.
	 */
	@Override
	public boolean areAllWordsPalindromes(String sentence) {
		//split sentence into words.
		ArrayList<String> split = split(sentence);
		//iterate over words.
		for (String word : split) {
			//check if single word is palindrome.
			if (!isPalindrome(word)) {
				//if at least one word is not a palindrome, return false.
				return false;
			}
		}
		//otherwise return true.
		return true;
	}

	/**
	 * Splits a sentence of words separated by a single whitespace into words.
	 * 
	 * @param sentence a sentence of words separated by a single whitespace
	 * @return the list of words that have been found.
	 */
	public ArrayList<String> split(String sentence) {
		//variable that contains the found words.
		ArrayList<String> words = new ArrayList<String>();
		//start index of the current word.
		int wordStart = 0;
		//iterate over all chars.
		for (int i = 0; i < sentence.length(); i++) {
			if (sentence.charAt(i) == ' ') {
				//if we found a whitespace, store the current word in the list.
				words.add(sentence.substring(wordStart, i));
				//update the word start.
				wordStart = i + 1;
			}
		}
		//insert the last word to the list.
		words.add(sentence.substring(wordStart, sentence.length()));
		return words;
	}

	/**
	 * Checks if a given word is a palindrome.
	 * 
	 * @param word a given word.
	 * @return true if the word is a palindrome.
	 */
	public boolean isPalindrome(String word) {
		//a word is a palindrome if it is still the same after it is reversed.
		return word.equals(reverse(word));
	}

	/**
	 * Returns the reversed version of the input word.
	 * 
	 * @param word a word.
	 * @return the reversed version of the input word.
	 */
	public String reverse(String word) {
		//StringBuilder for the reversed word.
		StringBuilder builder = new StringBuilder();
		//iterate over the characters of the word (from back to front).
		for (int i = word.length() - 1; i >= 0; i--) {
			//add the characters to the StringBuilder.
			builder.append(word.charAt(i));
		}
		//return the builders content.
		return builder.toString();
	}
}
