package de.unibi.citec.fit.javasamples.strategies;

import de.unibi.citec.fit.javasamples.SampleSolution;
import java.util.Stack;

/**
 * own method, string-based, iterative approach with comments.
 *
 * @author Benjamin Paassen - bpaassen(at)techfak.uni-bielefeld.de
 */
public class ImplAAB1 implements SampleSolution {

	/**
	 * Checks if all words of the given sentence are palindromes.
	 *
	 * @param sentence a sentence with words separated by a single whitespace.
	 * @return true if all words are palindromes and false otherwise.
	 */
	@Override
	public boolean areAllWordsPalindromes(String sentence) {
		//stack of chars of the current word.
		Stack<Character> characters = new Stack<Character>();
		//start index of the current word.
		int wordStart = 0;
		for (int i = 0; i < sentence.length(); i++) {
			//iterate over all characters of the sentence.
			Character character = sentence.charAt(i);
			//if we find a whitespace, cut there.
			if (character == ' ') {
				//check if the current word is a palindrome.
				if (!checkForPalindrome(characters, sentence, wordStart,
						(wordStart + i) / 2)) {
					//if not return false directly.
					return false;
				}
				//then update the word start index.
				wordStart = i + 1;
				//and clear the stack.
				characters.clear();
			} else {
				/* 
				 * if we are still within the word, push the current char onto 
				 * the stack.
				 */
				characters.push(character);
			}
		}
		//also check if the last word is a palindrome.
		return checkForPalindrome(characters, sentence, wordStart, (wordStart
				+ sentence.length() - 1) / 2);
	}

	/**
	 * Checks if the given word is a palindrome. The word is given by its
	 * reversed version in a stack, the sentence that contains the original
	 * version as well as a start and an end index of the word within the sentence.
	 * 
	 * @param stack Stack that contains the reversed version of the word.
	 * @param sentence The sentence that contains the original word.
	 * @param startIndex The start index of the word within the sentence.
	 * @param endIndex The end index of the word within the sentence.
	 * @return true if the word is a palindrome and false if it isn't.
	 */
	private boolean checkForPalindrome(Stack<Character> stack, String sentence,
			int startIndex, int endIndex) {
		//iterate over the word in the sentence.
		for (int j = startIndex; j <= endIndex; j++) {
			/* 
			 * check if the current char in the reversed word is equal to the
			 * current char in the original word. If it is not, return false.
			 * 
			 * At this point we assume that stack and original word have the
			 * same length.
			 */
			if (!stack.pop().equals(sentence.charAt(j))) {
				return false;
			}
		}
		//return true if all chars were equal.
		return true;
	}
}
