package de.unibi.citec.fit.javasamples.strategies;

import de.unibi.citec.fit.javasamples.SampleSolution;
import java.util.ArrayList;

/**
 * own method, array-based, split-first approach with comments.
 *
 * @author Benjamin Paassen - bpaassen(at)techfak.uni-bielefeld.de
 */
public class ImplABA1 implements SampleSolution {

	/**
	 * Checks if all words of the given sentence are palindromes.
	 *
	 * @param sentence a sentence with words separated by a single whitespace.
	 * @return true if all words are palindromes and false otherwise.
	 */
	@Override
	public boolean areAllWordsPalindromes(String sentence) {
		//split the sentence into its words.
		ArrayList<String> split = split(sentence);
		for (String word : split) {
			//then check if each word is a palindrome.
			if (!isPalindrome(word)) {
				//if not return false.
				return false;
			}
		}
		//otherwise return true.
		return true;
	}

	/**
	 * Splits a sentence of words separated by a single whitespace into words.
	 *
	 * @param sentence a sentence of words separated by a single whitespace
	 * @return the list of words that have been found.
	 */
	public ArrayList<String> split(String sentence) {
		//the output list of found words.
		ArrayList<String> words = new ArrayList<String>();
		//transform the sentence into an array of characters.
		char[] characters = sentence.toCharArray();
		//the current word start index.
		int wordStart = 0;
		//iterate over the sentence.
		for (int i = 0; i < characters.length; i++) {
			if (characters[i] == ' ') {
				//if we find a whitespace, store the current word.
				words.add(sentence.substring(wordStart, i));
				//and update the word start index.
				wordStart = i + 1;
			}
		}
		//also store the last word.
		words.add(sentence.substring(wordStart, sentence.length()));
		return words;
	}

	/**
	 * Checks if a given word is a palindrome.
	 *
	 * @param word a given word.
	 * @return true if the word is a palindrome.
	 */
	public boolean isPalindrome(String word) {
		//transform the input word to a char array.
		char[] characters = word.toCharArray();
		//iterate over the word by coming from both sides.
		for (int i = 0; i < characters.length / 2; i++) {
			/*
			 * check if the current char read from the end of the word equals
			 * the current char read from the beginning of the word.
			 */
			if (characters[i] != characters[characters.length - 1 - i]) {
				//if not: return false.
				return false;
			}
		}
		//if the word is the same read from backwards and forwards, return true.
		return true;
	}
}
