package de.unibi.citec.fit.javasamples.strategies;

import de.unibi.citec.fit.javasamples.SampleSolution;

/**
 * own method, array-based, iterative approach with comments.
 *
 * @author Benjamin Paassen - bpaassen(at)techfak.uni-bielefeld.de
 */
public class ImplABB1 implements SampleSolution {

	/**
	 * Checks if all words of the given sentence are palindromes.
	 *
	 * @param sentence a sentence with words separated by a single whitespace.
	 * @return true if all words are palindromes and false otherwise.
	 */
	@Override
	public boolean areAllWordsPalindromes(String sentence) {
		//transform the sentence into an array of characters.
		char[] characters = sentence.toCharArray();
		//the start index of the current word.
		int wordStart = 0;
		for (int i = 0; i < characters.length; i++) {
			//iterate over the whole sentence.
			if (characters[i] == ' ') {
				//if we found a whitespace, check if the current word is a palindrome.
				if (!checkForPalindrome(characters, wordStart, i - wordStart - 1)) {
					//if not return false.
					return false;
				}
				//update the start index of the current word.
				wordStart = i + 1;
			}
		}
		//check if the last word is a palindrome too.
		return checkForPalindrome(characters, wordStart, characters.length
				- wordStart - 1);
	}

	/**
	 * Checks if a given word is a palindrome. A word is characterized by the
	 * character array that contains the chars of the word, by the words start
	 * index within the array and the word length.
	 *
	 * @param characters a char array that contains the input word.
	 * @param startIndex the index of the words first char in the array.
	 * @param wordLength the words length.
	 * @return true if the word is a palindrome, false if it is not.
	 */
	private boolean checkForPalindrome(char[] characters,
			int startIndex, int wordLength) {
		// iterate over half of the word.
		for (int j = 0; j < wordLength / 2; j++) {
			//check if the word read backwards is the same as the word read forward.
			if (characters[startIndex + j] != characters[startIndex + wordLength
					- j]) {
				//if not return false.
				return false;
			}
		}
		//if so return true.
		return true;
	}
}