package de.unibi.citec.fit.javasamples.strategies;

import de.unibi.citec.fit.javasamples.SampleSolution;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * built-in, string-based, iterative approach with comments.
 *
 * @author Benjamin Paassen - bpaassen(at)techfak.uni-bielefeld.de
 */
public class ImplBAB1 implements SampleSolution {

	/**
	 * Pattern that identifies a word within the sentence.
	 */
	private static final Pattern wordPattern = Pattern.compile("\\w+");

	/**
	 * Checks if all words of the given sentence are palindromes.
	 *
	 * @param sentence a sentence with words separated by a single whitespace.
	 * @return true if all words are palindromes and false otherwise.
	 */
	@Override
	public boolean areAllWordsPalindromes(String sentence) {
		//Create a Java-Regex-Matcher for words in the sentence.
		Matcher matcher = wordPattern.matcher(sentence);
		//Find words with the matcher.
		while (matcher.find()) {
			//The current word is the complete match of the matcher.
			String word = matcher.group();
			/* 
			 * if the word is the same read backwards and forwards it is a 
			 * palindrome.
			 */
			if (!word.equals((new StringBuilder(word)).reverse().toString())) {
				//if it isn't, however, return false.
				return false;
			}
		}
		//if we got here all words were palindromes and we can return true.
		return true;
	}
}
