package de.unibi.citec.fit.javasamples.strategies;

import de.unibi.citec.fit.javasamples.SampleSolution;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * built-in, string-based, iterative approach with comments.
 *
 * @author Benjamin Paassen - bpaassen(at)techfak.uni-bielefeld.de
 */
public class ImplBBB1 implements SampleSolution {

	/**
	 * A pattern to identify words in a sentence.
	 */
	private static final Pattern wordPattern = Pattern.compile("\\w+");

	/**
	 * Checks if all words of the given sentence are palindromes.
	 *
	 * @param sentence a sentence with words separated by a single whitespace.
	 * @return true if all words are palindromes and false otherwise.
	 */
	@Override
	public boolean areAllWordsPalindromes(String sentence) {
		//Create a java regex matcher for words in the given sentence.
		Matcher matcher = wordPattern.matcher(sentence);
		//iterate over all words.
		while (matcher.find()) {
			/*
			 * A word is one complete match of the created matcher. We transform
			 * the resulting string to a character array.
			 */
			char[] characters = matcher.group().toCharArray();
			//iterate over half of the words length.
			for (int i = 0; i < characters.length / 2; i++) {
				/*
				 * Check if the word read backwards equals the word read
				 * forwards. If so the word is a palindrome.
				 */
				if (characters[i] != characters[characters.length - 1 - i]) {
					//if not return false.
					return false;
				}
			}
		}
		//if we got here all words are palindromes and we can return true.
		return true;
	}
}
