/* 
 * EDM Demo 2015
 * Educational Datamining Demonstrator for the TCS Alignment Toolbox
 * 
 * Copyright (C) 2015
 * Benjamin Paaßen
 * AG Theoretical Computer Science
 * Centre of Excellence Cognitive Interaction Technology (CITEC)
 * University of Bielefeld
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.citec.tcs.edm2015;

import java.awt.Desktop;
import java.awt.Dimension;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.net.URISyntaxException;
import javax.swing.BoxLayout;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

/**
 *
 * @author Benjamin Paassen - bpaassen(at)techfak.uni-bielefeld.de
 */
public abstract class Card extends JPanel {

	public static final int PREFERRED_WIDTH = 1024;
	public static final int PREFERRED_HEIGHT = 600;

	private final String title;
	private final String htmlFile;
	private final String description;

	public Card(String title, String htmlFile) {
		super();
		this.title = title;
		this.htmlFile = htmlFile;
		this.description = getTextFromFile("html", htmlFile);
		setup();
	}

	private void setup() {
		this.setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));

		final JEditorPane descriptionPane = new JEditorPane("text/html", getDescription());
		descriptionPane.setEditable(false);
		descriptionPane.addHyperlinkListener(new HyperlinkListener() {
			@Override
			public void hyperlinkUpdate(HyperlinkEvent e) {
				if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
					if (Desktop.isDesktopSupported()) {
						try {
							Desktop.getDesktop().browse(e.getURL().toURI());
						} catch (IOException | URISyntaxException ex) {
							throw new RuntimeException(ex);
						}
					}
				}
			}
		});

		JScrollPane descriptionScroll = new JScrollPane(descriptionPane);
		final Dimension cardSize = getPreferredSize();
		final Dimension halfSize = new Dimension(
				(int) (0.4 * cardSize.width), (int) (0.9 * cardSize.height));
		descriptionScroll.setPreferredSize(halfSize);
		descriptionScroll.setMinimumSize(halfSize);
		add(descriptionScroll);

		final JPanel rightPanel = new JPanel();
		rightPanel.setPreferredSize(halfSize);
		rightPanel.setMinimumSize(halfSize);
		fillRightPanel(rightPanel);
		add(rightPanel);

		// set cursor back to beginning
		descriptionPane.setCaretPosition(0);
	}

	public abstract void fillRightPanel(JPanel panel);

	public String getTitle() {
		return title;
	}

	public String getHtmlFile() {
		return htmlFile;
	}

	public String getDescription() {
		return description;
	}

	public void addAsTabTo(JTabbedPane tabs) {
		tabs.addTab(title, this);
	}

	@Override
	public Dimension getPreferredSize() {
		return new Dimension(PREFERRED_WIDTH, PREFERRED_HEIGHT);
	}

	public static String getTextFromFile(String dir, String file) {
		try {
			final BufferedReader in = new BufferedReader(new FileReader(dir + "/" + file));
			String line;
			final StringBuilder buf = new StringBuilder();
			boolean first = true;
			while ((line = in.readLine()) != null) {
				if (first) {
					first = false;
				} else {
					buf.append("\n");
				}
				buf.append(line);
			}
			in.close();
			return buf.toString();
		} catch (IOException ex) {
			throw new RuntimeException(ex);
		}
	}
}
