/* 
 * EDM Demo 2015
 * Educational Datamining Demonstrator for the TCS Alignment Toolbox
 * 
 * Copyright (C) 2015
 * Benjamin Paaßen
 * AG Theoretical Computer Science
 * Centre of Excellence Cognitive Interaction Technology (CITEC)
 * University of Bielefeld
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.citec.tcs.edm2015;

import de.citec.tcs.alignment.comparators.GapComparator;
import de.citec.tcs.alignment.sequence.StringValue;
import de.citec.tcs.alignment.sequence.ValueType;

/**
 *
 * @author Benjamin Paassen - bpaassen(at)techfak.uni-bielefeld.de
 */
public class ScopeComparator implements GapComparator<StringValue> {

	@Override
	public double delete(StringValue a) {
		return 1;
	}

	@Override
	public double insert(StringValue b) {
		return 1;
	}

	@Override
	public ValueType getType() {
		return ValueType.STRING;
	}

	@Override
	public double compare(StringValue a, StringValue b) {
		if (b == null) {
			if (a == null) {
				return 0;
			} else {
				return delete(a);
			}
		}
		if (a == null) {
			return insert(b);
		}
		// get the int array representation of both scopes
		final int[] left = parseArray(a.getString());
		final int[] right = parseArray(b.getString());
		// retrieve the shorter one of both scopes
		final int[] shorter;
		final int[] longer;
		if (left.length > right.length) {
			longer = left;
			shorter = right;
		} else {
			longer = right;
			shorter = left;
		}
		if(longer.length == 0){
			return 0;
		}
		// count the number of positions, where both scopes are equal.
		int eq = 0;
		for (int i = 0; i < shorter.length; i++) {
			if (shorter[i] == longer[i]) {
				eq++;
			}
		}
		// return it, normalized by the length of the longer sequence.
		return 1 - ((double) eq / (double) longer.length);
	}

	private static int[] parseArray(String arr) {
		final String inner = arr.substring(1, arr.length() - 1);
		if (inner.isEmpty()) {
			return new int[0];
		}
		final String[] numberStrs = inner.split(",\\s*");
		final int[] out = new int[numberStrs.length];
		for (int i = 0; i < out.length; i++) {
			out[i] = Integer.parseInt(numberStrs[i]);
		}
		return out;
	}

}
