/* 
 * EDM Demo 2015
 * Educational Datamining Demonstrator for the TCS Alignment Toolbox
 * 
 * Copyright (C) 2015
 * Benjamin Paaßen
 * AG Theoretical Computer Science
 * Centre of Excellence Cognitive Interaction Technology (CITEC)
 * University of Bielefeld
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.citec.tcs.edm2015;

import de.sciss.syntaxpane.DefaultSyntaxKit;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FilenameFilter;
import java.util.Arrays;
import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

/**
 *
 * @author Benjamin Paassen - bpaassen(at)techfak.uni-bielefeld.de
 */
public class Step1Card extends Card implements ActionListener {

	private static final String START_PROGRAM = "ImplBAA.java";
	private JEditorPane codePane;

	public Step1Card() {
		super("Programming Task", "step1.html");
	}

	@Override
	public void fillRightPanel(JPanel codePanel) {
		codePanel.setLayout(new BoxLayout(codePanel, BoxLayout.PAGE_AXIS));
		final Dimension size = codePanel.getPreferredSize();

		final String[] codeFileNames = (new File("programs")).list(new ImplFilter());
		Arrays.sort(codeFileNames);
		final JComboBox comboBox = new JComboBox(codeFileNames);
		comboBox.setSelectedIndex(Arrays.binarySearch(codeFileNames, START_PROGRAM));
		comboBox.addActionListener(this);
		codePanel.add(comboBox);

		// load syntax highlighting.
		DefaultSyntaxKit.initKit();

		this.codePane = new JEditorPane("text/java", Card.getTextFromFile("programs", START_PROGRAM));
		codePane.setEditable(false);
		final JScrollPane codeScroll = new JScrollPane(codePane);

		final Dimension codeScrollSize = new Dimension(size.width, (int) (0.95 * size.height));
		codeScroll.setPreferredSize(codeScrollSize);
		codeScroll.setMinimumSize(codeScrollSize);
		codePanel.add(codeScroll);

		// set cursor back to top.
		codePane.setCaretPosition(0);
	}

	private static class ImplFilter implements FilenameFilter {

		@Override
		public boolean accept(File dir, String name) {
			return name.matches("Impl[AB]{3}\\d?\\.java");
		}

	}

	@Override
	public void actionPerformed(ActionEvent e) {
		JComboBox cb = (JComboBox) e.getSource();
		final String filename = (String) cb.getSelectedItem();
		codePane.setText(Card.getTextFromFile("programs", filename));
		codePane.setCaretPosition(0);
	}
}
