/* 
 * EDM Demo 2015
 * Educational Datamining Demonstrator for the TCS Alignment Toolbox
 * 
 * Copyright (C) 2015
 * Benjamin Paaßen
 * AG Theoretical Computer Science
 * Centre of Excellence Cognitive Interaction Technology (CITEC)
 * University of Bielefeld
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.citec.tcs.edm2015;

import de.citec.tcs.alignment.sequence.Node;
import de.citec.tcs.alignment.sequence.NodeSpecification;
import de.citec.tcs.alignment.sequence.Sequence;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.TreeMap;
import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

/**
 *
 * @author Benjamin Paassen - bpaassen(at)techfak.uni-bielefeld.de
 */
public class Step2Card extends Card implements ActionListener {

	public static final TreeMap<String, Sequence> programSequences = readPrograms();
	private JEditorPane sequencePane;

	public Step2Card() {
		super("Sequences", "step2.html");
	}

	@Override
	public void fillRightPanel(JPanel sequencePanel) {
		sequencePanel.setLayout(new BoxLayout(sequencePanel, BoxLayout.PAGE_AXIS));
		final Dimension size = sequencePanel.getPreferredSize();

		final String[] fogFileNames = programSequences.keySet().toArray(new String[0]);
		final JComboBox comboBox = new JComboBox(fogFileNames);
		comboBox.setSelectedIndex(0);
		comboBox.addActionListener(this);
		sequencePanel.add(comboBox);

		try {
			this.sequencePane = new JEditorPane("text", sequenceToJSON(
					FOGSequenceReader.readFOG("fog/" + fogFileNames[0])));
		} catch (IOException ex) {
			throw new RuntimeException(ex);
		}
		sequencePane.setEditable(false);
		JScrollPane sequenceScroll = new JScrollPane(sequencePane);

		final Dimension sequenceScrollSize = new Dimension(size.width, (int) (0.95 * size.height));
		sequenceScroll.setPreferredSize(sequenceScrollSize);
		sequenceScroll.setMinimumSize(sequenceScrollSize);
		sequencePanel.add(sequenceScroll);

		sequencePane.setCaretPosition(0);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		JComboBox cb = (JComboBox) e.getSource();
		final String filename = (String) cb.getSelectedItem();
		sequencePane.setText(sequenceToJSON(programSequences.get(filename)));
		sequencePane.setCaretPosition(0);
	}

	private static TreeMap<String, Sequence> readPrograms() {
		final TreeMap<String, Sequence> out = new TreeMap<>();
		final String[] filenames = (new File("fog")).list(new ImplFilter());
		for (final String f : filenames) {
			try {
				final Sequence seq = FOGSequenceReader.readFOG("fog/" + f);
				out.put(f, seq);
			} catch (IOException ex) {
				throw new RuntimeException(ex);
			}
		}
		return out;
	}

	private static class ImplFilter implements FilenameFilter {

		@Override
		public boolean accept(File dir, String name) {
			return name.matches("Impl[AB]{3}\\d?\\.fog");
		}

	}

	public static String sequenceToJSON(Sequence seq) {
		final NodeSpecification spec = seq.getNodeSpecification();
		final StringBuilder buf = new StringBuilder();
		buf.append("[");
		boolean first = true;
		for (final Node n : seq.getNodes()) {
			if (first) {
				first = false;
			} else {
				buf.append(",");
			}
			buf.append(System.getProperty("line.separator"));
			buf.append("{");
			buf.append(System.getProperty("line.separator"));
			for (int k = 0; k < spec.size(); k++) {
				buf.append("\t\"");
				buf.append(spec.getKeyword(k));
				buf.append("\" : \"");
				if (n.getValue(k) != null) {
					buf.append(n.getValue(k).toString());
				}
				buf.append("\"");
				if (k < spec.size() - 1) {
					buf.append(",");
				}
				buf.append(System.getProperty("line.separator"));
			}
			buf.append("}");
		}
		buf.append(System.getProperty("line.separator"));
		buf.append("]");
		return buf.toString();
	}
}
