/* 
 * EDM Demo 2015
 * Educational Datamining Demonstrator for the TCS Alignment Toolbox
 * 
 * Copyright (C) 2015
 * Benjamin Paaßen
 * AG Theoretical Computer Science
 * Centre of Excellence Cognitive Interaction Technology (CITEC)
 * University of Bielefeld
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.citec.tcs.edm2015;

import de.citec.tcs.alignment.AlignmentPath;
import de.citec.tcs.alignment.AlignmentSpecification;
import de.citec.tcs.alignment.Operation;
import de.citec.tcs.alignment.StrictAlignmentFullAlgorithm;
import de.citec.tcs.alignment.comparators.OperationType;
import de.citec.tcs.alignment.sequence.Sequence;
import de.citec.tcs.alignment.wrappers.StringEditDistance;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

/**
 *
 * @author Benjamin Paassen - bpaassen(at)techfak.uni-bielefeld.de
 */
public class Step3Card extends Card implements ActionListener {

	private JTextField leftField;
	private JTextField rightField;
	private JEditorPane alignmentPane;

	public Step3Card() {
		super("Alignment", "step3.html");
	}

	@Override
	public void fillRightPanel(JPanel alignmentPanel) {
		alignmentPanel.setLayout(new BoxLayout(alignmentPanel, BoxLayout.PAGE_AXIS));
		final Dimension size = alignmentPanel.getPreferredSize();

		this.leftField = new JTextField("airplane");
		this.leftField.setMinimumSize(new Dimension(size.width, (int) (0.03 * size.height)));
		alignmentPanel.add(leftField);
		this.rightField = new JTextField("darling");
		this.rightField.setMinimumSize(new Dimension(size.width, (int) (0.03 * size.height)));
		alignmentPanel.add(rightField);
		final JButton button = new JButton("Align");
		button.addActionListener(this);
		alignmentPanel.add(button);

		this.alignmentPane = new JEditorPane("text", "");
		alignmentPane.setEditable(false);
		JScrollPane alignmentScroll = new JScrollPane(alignmentPane);

		final Dimension alignmentScrollSize = new Dimension(size.width, (int) (0.9 * size.height));
		alignmentScroll.setPreferredSize(alignmentScrollSize);
		alignmentScroll.setMinimumSize(alignmentScrollSize);
		alignmentPanel.add(alignmentScroll);

		alignmentPane.setCaretPosition(0);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		final String left = leftField.getText();
		final String right = rightField.getText();
		final Sequence[] seqs = StringEditDistance.toSequences(new String[]{left, right});
		final AlignmentSpecification spec = StringEditDistance.setUpSpecification(seqs);
		final StrictAlignmentFullAlgorithm algo = new StrictAlignmentFullAlgorithm(spec);
		final AlignmentPath path = algo.calculateAlignment(seqs[0], seqs[1]);
		alignmentPane.setText(editDistanceString(path));
		alignmentPane.setCaretPosition(0);
	}

	private static String editDistanceString(AlignmentPath path) {
		final StringBuilder buf = new StringBuilder();
		int total = 0;
		for (final Operation op : path.getOperations()) {
			String left = "-";
			switch (op.getType()) {
				case DELETION:
				case REPLACEMENT:
					left = op.getLeft().getValue(0).toString();
			}
			buf.append(left);
			buf.append("\t");
			String right = "-";
			switch (op.getType()) {
				case INSERTION:
				case REPLACEMENT:
					right = op.getRight().getValue(0).toString();
			}
			buf.append(right);
			buf.append("\t");
			buf.append(op.getType().toString().toLowerCase());
			if (op.getType() == OperationType.REPLACEMENT) {
				if (left.equals(right)) {
					buf.append(" (match)");
				} else {
					buf.append(" (mismatch)");
				}
			}
			buf.append(System.getProperty("line.separator"));
			total += op.getWeightedLocalCost();
		}
		buf.append("overall distance = ");
		buf.append(Integer.toString(total));
		return buf.toString();
	}
}
